@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
-from uamobile import exceptions
 from uamobile.base import UserAgent, Display
 
 class NonMobileUserAgent(UserAgent):
     name = 'NonMobile'
     carrier = 'NonMobile'
     short_carrier = 'N'
+    serialnumber = None
 
     def get_name(self):
         return 'NonMobile'
@@ -16,13 +16,8 @@ class NonMobileUserAgent(UserAgent):
     def supports_cookie(self):
         return True
 
-    def parse(self):
-        pass
-
     def make_display(self):
         """
         create a new Display object.
         """
         return Display()
-
-    serialnumber = property(lambda _: None)
